@@ -9,7 +9,7 @@ from contextlib import contextmanager
 import numpy as np
 
 import astropy.units as u
-from astropy.coordinates import Attribute, ConvertError
+from astropy.coordinates import ConvertError, QuantityAttribute
 from astropy.coordinates.baseframe import BaseCoordinateFrame, RepresentationMapping
 from astropy.coordinates.representation import (
     CartesianDifferential,
@@ -87,6 +87,11 @@ def _frame_parameters():
                        "        it must be a solar system body that can be parsed by\n"
                        "        `~sunpy.coordinates.ephemeris.get_body_heliographic_stonyhurst`\n"
                        "        at the time ``obstime``. Defaults to Earth center.")
+    ret['rsun'] = ("rsun : `~astropy.units.Quantity`\n"
+                   "        The radius of the Sun in length units. Used to convert a 2D\n"
+                   "        coordinate (i.e., no ``radius`` component) to a 3D coordinate by\n"
+                   "        assuming that the coordinate is on the surface of the Sun. Defaults\n"
+                   "        to the photospheric radius as defined in `sunpy.sun.constants`.")
     ret['equinox'] = (f"equinox : {_variables_for_parse_time_docstring()['parse_time_types']}\n"
                       "        The date for the mean vernal equinox.\n"
                       "        Defaults to the J2000.0 equinox.")
@@ -101,8 +106,8 @@ class SunPyBaseCoordinateFrame(BaseCoordinateFrame):
     This class is not intended to be used directly and has no transformations defined.
 
     * Defines the frame attribute ``obstime`` for observation time.
-    * Defines a default longitude wrap angle of 180 degrees, which can be overridden via the class
-      variable ``_wrap_angle``.
+    * Defines a default wrap angle of 180 degrees for longitude in spherical coordinates,
+      which can be overridden via the class variable ``_wrap_angle``.
     * Inject a nice way of representing the object which the coordinate represents.
     """
     obstime = TimeFrameAttributeSunPy()
@@ -116,7 +121,7 @@ class SunPyBaseCoordinateFrame(BaseCoordinateFrame):
                                 RepresentationMapping('d_distance', 'd_distance', u.km/u.s)],
     }
 
-    _wrap_angle = 180*u.deg
+    _wrap_angle = 180*u.deg  # for longitude in spherical coordinates
 
     def __init__(self, *args, **kwargs):
         self.object_name = None
@@ -134,20 +139,17 @@ class SunPyBaseCoordinateFrame(BaseCoordinateFrame):
         return
 
     def represent_as(self, base, s='base', in_frame_units=False):
-        """
-        If a frame wrap angle is set, use that wrap angle for any spherical representations.
-        """
         data = super().represent_as(base, s, in_frame_units=in_frame_units)
+
+        # If a frame wrap angle is set, use that wrap angle for any spherical representations.
         if self._wrap_angle is not None and \
            isinstance(data, (UnitSphericalRepresentation, SphericalRepresentation)):
             data.lon.wrap_angle = self._wrap_angle
         return data
 
     def __str__(self):
-        """
-        We override this here so that when you print a SkyCoord it shows the
-        observer as the string and not the whole massive coordinate.
-        """
+        # We override this here so that when you print a SkyCoord it shows the
+        # observer as the string and not the whole massive coordinate.
         if getattr(self, "object_name", None):
             return f"<{self.__class__.__name__} Coordinate for '{self.object_name}'>"
         else:
@@ -169,17 +171,33 @@ class BaseHeliographic(SunPyBaseCoordinateFrame):
                                 RepresentationMapping('d_distance', 'd_radius', u.km/u.s)],
     }
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+    rsun = QuantityAttribute(default=_RSUN, unit=u.km)
 
-        self._make_3d()
+    def make_3d(self):
+        """
+        Returns a fully 3D coordinate based on this coordinate.
 
-    def _make_3d(self):
-        # Make 3D if specified as 2D
+        If this coordinate is only 2D (i.e., no ``radius`` component) or is a
+        unit vector (i.e., the norm of the coordinate is unity), a new
+        coordinate is created that corresponds to the surface of the Sun.
+        That is, the 3D coordinate will retain the ``lon`` and ``lat``, and
+        ``radius`` will be set to the frame's ``rsun`` frame attribute.
+
+        If this coordinate is already fully 3D, it is directly returned, even
+        if it does not lie on the surface of the Sun.
+
+        Returns
+        -------
+        frame : `~sunpy.coordinates.frames.BaseHeliographic`
+            The fully 3D coordinate
+        """
+        # Check if the coordinate is 2D
         if (self._data is not None and self._data.norm().unit is u.one
                 and u.allclose(self._data.norm(), 1*u.one)):
+            return self.realize_frame(self._data * self.rsun)
 
-            self._data *= _RSUN.to(u.km)
+        # The coordinate is already 3D
+        return self
 
 
 @add_common_docstring(**_frame_parameters())
@@ -208,6 +226,7 @@ class HeliographicStonyhurst(BaseHeliographic):
     {data}
     {lonlat}
     {radius}
+    {rsun}
     {common}
 
     Examples
@@ -219,26 +238,21 @@ class HeliographicStonyhurst(BaseHeliographic):
     ...               frame="heliographic_stonyhurst",
     ...               obstime="2010/01/01T00:00:45")
     >>> sc
-    <SkyCoord (HeliographicStonyhurst: obstime=2010-01-01T00:00:45.000): (lon, lat, radius) in (deg, deg, km)
+    <SkyCoord (HeliographicStonyhurst: obstime=2010-01-01T00:00:45.000, rsun=695700.0 km): (lon, lat, radius) in (deg, deg, km)
         (1., 1., 2.)>
     >>> sc.frame
-    <HeliographicStonyhurst Coordinate (obstime=2010-01-01T00:00:45.000): (lon, lat, radius) in (deg, deg, km)
+    <HeliographicStonyhurst Coordinate (obstime=2010-01-01T00:00:45.000, rsun=695700.0 km): (lon, lat, radius) in (deg, deg, km)
         (1., 1., 2.)>
     >>> sc = SkyCoord(HeliographicStonyhurst(-10*u.deg, 2*u.deg))
     >>> sc
-    <SkyCoord (HeliographicStonyhurst: obstime=None): (lon, lat, radius) in (deg, deg, km)
-        (-10., 2., 695700.)>
+    <SkyCoord (HeliographicStonyhurst: obstime=None, rsun=695700.0 km): (lon, lat) in deg
+        (-10., 2.)>
     >>> sc = SkyCoord(CartesianRepresentation(0*u.km, 45*u.km, 2*u.km),
     ...               obstime="2011/01/05T00:00:50",
     ...               frame="heliographic_stonyhurst")
     >>> sc
-    <SkyCoord (HeliographicStonyhurst: obstime=2011-01-05T00:00:50.000): (lon, lat, radius) in (deg, deg, km)
+    <SkyCoord (HeliographicStonyhurst: obstime=2011-01-05T00:00:50.000, rsun=695700.0 km): (lon, lat, radius) in (deg, deg, km)
     (90., 2.54480438, 45.04442252)>
-
-    Notes
-    -----
-    This frame will always be converted a 3D frame where the radius defaults to
-    ``rsun``.
     """
     name = "heliographic_stonyhurst"
 
@@ -275,6 +289,7 @@ class HeliographicCarrington(BaseHeliographic):
     {lonlat}
     {radius}
     {observer}
+    {rsun}
     {common}
 
     Examples
@@ -287,7 +302,7 @@ class HeliographicCarrington(BaseHeliographic):
     ...               observer="earth",
     ...               obstime="2010/01/01T00:00:30")
     >>> sc
-    <SkyCoord (HeliographicCarrington: obstime=2010-01-01T00:00:30.000, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (lon, lat, radius) in (deg, deg, km)
+    <SkyCoord (HeliographicCarrington: obstime=2010-01-01T00:00:30.000, rsun=695700.0 km, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (lon, lat, radius) in (deg, deg, km)
         (1., 2., 3.)>
 
     >>> sc = SkyCoord([1,2,3]*u.deg, [4,5,6]*u.deg, [5,6,7]*u.km,
@@ -295,14 +310,14 @@ class HeliographicCarrington(BaseHeliographic):
     ...               observer="self",
     ...               frame="heliographic_carrington")
     >>> sc
-    <SkyCoord (HeliographicCarrington: obstime=2010-01-01T00:00:45.000, observer=self): (lon, lat, radius) in (deg, deg, km)
+    <SkyCoord (HeliographicCarrington: obstime=2010-01-01T00:00:45.000, rsun=695700.0 km, observer=self): (lon, lat, radius) in (deg, deg, km)
         [(1., 4., 5.), (2., 5., 6.), (3., 6., 7.)]>
 
     >>> sc = SkyCoord(CartesianRepresentation(0*u.km, 45*u.km, 2*u.km),
     ...               obstime="2011/01/05T00:00:50",
     ...               frame="heliographic_carrington")
     >>> sc
-    <SkyCoord (HeliographicCarrington: obstime=2011-01-05T00:00:50.000, observer=None): (lon, lat, radius) in (deg, deg, km)
+    <SkyCoord (HeliographicCarrington: obstime=2011-01-05T00:00:50.000, rsun=695700.0 km, observer=None): (lon, lat, radius) in (deg, deg, km)
         (90., 2.54480438, 45.04442252)>
     """
     name = "heliographic_carrington"
@@ -375,6 +390,14 @@ class Heliocentric(SunPyBaseCoordinateFrame):
 
     observer = ObserverCoordinateAttribute(HeliographicStonyhurst)
 
+    def represent_as(self, base, s='base', in_frame_units=False):
+        data = super().represent_as(base, s, in_frame_units=in_frame_units)
+
+        # For cylindrical representations, wrap the `psi` component (natively `phi`) at 360 deg
+        if isinstance(data, CylindricalRepresentation):
+            data.phi.wrap_at(360*u.deg, inplace=True)
+        return data
+
 
 @add_common_docstring(**_frame_parameters())
 class Helioprojective(SunPyBaseCoordinateFrame):
@@ -409,10 +432,7 @@ class Helioprojective(SunPyBaseCoordinateFrame):
         The distance coordinate from the observer for this object.
         Not needed if ``data`` is given.
     {observer}
-    rsun : `~astropy.units.Quantity`
-        The physical (length) radius of the Sun. Used to calculate the position
-        of the limb for calculating distance from the observer to the
-        coordinate. Defaults to the solar radius.
+    {rsun}
     {common}
 
     Examples
@@ -447,7 +467,7 @@ class Helioprojective(SunPyBaseCoordinateFrame):
                                       RepresentationMapping('lat', 'Ty', u.arcsec)],
     }
 
-    rsun = Attribute(default=_RSUN.to(u.km))
+    rsun = QuantityAttribute(default=_RSUN, unit=u.km)
     observer = ObserverCoordinateAttribute(HeliographicStonyhurst)
 
     def make_3d(self):
